// MRListCt.cpp : implementation file
//
// MultiRen version 1.0
// Copyright  1998 Ziff-Davis, Inc.
// First published in PC Magazine, US Edition
// June 9, 1998
//
// Written by Gregory A. Wolking

#include "stdafx.h"
#include "MultiRen.h"
#include "MRListCt.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMRListCtrl

CMRListCtrl::CMRListCtrl()
{
	m_bIsDragging = FALSE;
	m_intDragDest = m_intDragSource = -1;
	m_pDragImage = NULL;
}

CMRListCtrl::~CMRListCtrl()
{
	if (m_pDragImage != NULL)
		delete m_pDragImage;
}


BEGIN_MESSAGE_MAP(CMRListCtrl, CListCtrl)
	//{{AFX_MSG_MAP(CMRListCtrl)
	ON_NOTIFY_REFLECT(LVN_BEGINDRAG, OnBegindrag)
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMRListCtrl message handlers

void CMRListCtrl::OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	CPoint ptDrag;
	int iItem;

	SetFocus();
	ptDrag = pNMListView->ptAction;
	iItem = pNMListView->iItem;
	m_bIsDragging = TRUE;
	m_intDragSource = iItem;
	m_intDragDest = -1;
	m_pDragImage = CreateDragImage(iItem, &ptDrag);
	m_pDragImage->DragShowNolock(TRUE);
	m_pDragImage->SetDragCursorImage(0, CPoint(0, 0));
	m_pDragImage->BeginDrag(0, CPoint(0,0));
	m_pDragImage->DragMove(ptDrag);
	m_pDragImage->DragEnter(this, ptDrag);
	SetCapture();
	*pResult = 0;
}

void CMRListCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	UINT nMyFlags;
	UINT nData;
	CPoint ptThis = point;
	
	if (m_bIsDragging)
	{
		m_intDragDest = HitTest(ptThis, &nMyFlags);
		m_pDragImage->DragLeave(this);
		m_pDragImage->EndDrag();
		delete m_pDragImage;
		m_pDragImage = NULL;
		ReleaseCapture();
		m_bIsDragging = FALSE;
		if (m_intDragDest != -1)
		{
			if (m_intDragSource != m_intDragDest)
			{
				nData = GetItemData(m_intDragSource);
				DeleteItem(m_intDragSource);
				if (m_intDragSource < m_intDragDest)
					--m_intDragDest;
				InsertItem(LVIF_TEXT | LVIF_PARAM | LVIF_STATE,
					m_intDragDest + 1, LPSTR_TEXTCALLBACK,
					LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED, 0, nData); 
				EnsureVisible(m_intDragDest + 1, FALSE);
				::PostMessage(m_hWndParent, WM_MY_ITEM_MOVED, 0, 0);
			}
			else
			{
				EnsureVisible(m_intDragSource, FALSE);
				SetItem(m_intDragSource, 0, LVIF_STATE, NULL, 0, LVIS_SELECTED | LVIS_FOCUSED,
					LVIS_SELECTED | LVIS_FOCUSED, 0);
			}
		}
	}
	CListCtrl::OnLButtonUp(nFlags, point);
}

void CMRListCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	int iItem;
	UINT nMyFlags;
	CSize size(0, 0);
	CPoint ptAnchor, ptCurrent, ptThis;
	MSG msg;

	ptThis = point;
	if (m_bIsDragging)
	{
		m_pDragImage->DragMove(point);
		iItem = HitTest(ptThis, &nMyFlags);
		if (iItem == -1)
			SetCursor(LoadCursor(NULL, MAKEINTRESOURCE(IDC_NO)));
		else
			SetCursor(LoadCursor(NULL, MAKEINTRESOURCE(IDC_ARROW)));
		if (nMyFlags & LVHT_ABOVE)
		{
			size.cy = -16;
		}
		else if (nMyFlags & LVHT_BELOW)
		{
			size.cy = 16;
		}
		else if (iItem != -1)
		{
			m_pDragImage->DragLeave(this);
			SetItem(iItem, 0, LVIF_STATE, NULL, 0, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED, 0); 
			UpdateWindow();
			m_pDragImage->DragEnter(this, point);
		}
		if (size.cy)
		{
			ptAnchor = point;
			do
			{
				m_pDragImage->DragLeave(this);
				Scroll(size);
				UpdateWindow();
				m_pDragImage->DragEnter(this, point);
				Sleep(100);
				GetCursorPos(&ptCurrent);
				ScreenToClient(&ptCurrent);
				if (PeekMessage(&msg, NULL, WM_MOUSEFIRST, WM_MOUSELAST, PM_NOREMOVE))
				{
					if (msg.message == WM_LBUTTONUP || msg.message == WM_RBUTTONDOWN)
						break;
				}
			} while (ptAnchor == ptCurrent);
		}
	}
	CListCtrl::OnMouseMove(nFlags, point);
}

void CMRListCtrl::OnRButtonDown(UINT nFlags, CPoint point) 
{
	if (m_bIsDragging)
		Cancel_Drag();
	else
		CListCtrl::OnRButtonDown(nFlags, point);
}

void CMRListCtrl::Cancel_Drag()
{
	m_bIsDragging = FALSE;				// Reset flag.
	m_pDragImage->DragLeave(this);		// Erase drag image from screen.
	m_pDragImage->EndDrag();			// End the drag.
	delete m_pDragImage;				// Delete the drag image.
	m_pDragImage = NULL;				// NULL its pointer.
	ReleaseCapture();					// Release mouse capture.
}

