// RenDlg.cpp : implementation file
//
// MultiRen version 1.0
// Copyright  1998 Ziff-Davis, Inc.
// First published in PC Magazine, US Edition
// June 9, 1998
//
// Written by Gregory A. Wolking

#include "stdafx.h"
#include <math.h>
#include "MultiRen.h"
#include "RenDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Dialog box flags for final confirmation prompt.
#define VERSION_MESSAGE_STRING "MultiRen Version 1.0\n"

/////////////////////////////////////////////////////////////////////////////
// CRenDlg dialog


CRenDlg::CRenDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRenDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRenDlg)
	m_bUndo = FALSE;
	//}}AFX_DATA_INIT
	m_hIcon = (HICON) LoadImage(AfxGetApp()->m_hInstance, MAKEINTRESOURCE(IDI_ICON1), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR); 
	m_pFileNames = NULL;
	m_pFileExtensions = NULL;
	m_pNewNames = NULL;
	m_nFileCount = 0;
	m_bExtAscending = TRUE;
	m_bNameAscending = TRUE;
	m_nMinDigits = 1;
	m_intDigits = 2;
	m_bNewNamesValid = FALSE;
	m_bSettingsChanged = FALSE;
	m_hAccelTable = NULL;
}


void CRenDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRenDlg)
	DDX_Control(pDX, IDC_cmdSaveSettings, m_cmdSaveSettings);
	DDX_Control(pDX, IDC_lblWarning, m_lblWarning);
	DDX_Control(pDX, IDC_spnDigits, m_spnDigits);
	DDX_Control(pDX, IDC_txtStart, m_txtStart);
	DDX_Control(pDX, IDC_txtExtTemplate, m_txtExtTemplate);
	DDX_Control(pDX, IDC_txtDigits, m_txtDigits);
	DDX_Control(pDX, IDC_txtBaseTemplate, m_txtBaseTemplate);
	DDX_Control(pDX, IDC_cmdExecute, m_cmdExecute);
	DDX_Control(pDX, IDC_lstFiles, m_lstFiles);
	DDX_Check(pDX, IDC_chkUndo, m_bUndo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRenDlg, CDialog)
	//{{AFX_MSG_MAP(CRenDlg)
	ON_NOTIFY(LVN_GETDISPINFO, IDC_lstFiles, OnGetdispinfolstFiles)
	ON_NOTIFY(LVN_COLUMNCLICK, IDC_lstFiles, OnColumnclicklstFiles)
	ON_NOTIFY(UDN_DELTAPOS, IDC_spnDigits, OnDeltaposspnDigits)
	ON_BN_CLICKED(IDC_cmdExecute, OncmdExecute)
	ON_EN_CHANGE(IDC_txtBaseTemplate, OnChangetxtBaseTemplate)
	ON_BN_CLICKED(IDC_cmdSaveSettings, OncmdSaveSettings)
	ON_WM_CONTEXTMENU()
	ON_WM_GETMINMAXINFO()
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_cmdAbout, OncmdAbout)
	ON_BN_CLICKED(IDC_cmdPreview, OnOK)
	ON_COMMAND(ID_MOVE_UP, OnMoveUp)
	ON_COMMAND(ID_MOVE_DOWN, OnMoveDown)
	//}}AFX_MSG_MAP
	// NOTE: EN_CHANGE messages from the Extension, Digits, and Start
	// fields are all mapped to the same handler used for the Base Name field.
	ON_EN_CHANGE(IDC_txtExtTemplate, OnChangetxtBaseTemplate)
	ON_EN_CHANGE(IDC_txtStart, OnChangetxtBaseTemplate)
	ON_EN_CHANGE(IDC_txtDigits, OnChangetxtBaseTemplate)
	// Handler for custom message posted by the list control.
	ON_MESSAGE(WM_MY_ITEM_MOVED, OnMyItemMoved)
	// Handler for custom command generated by "What's this?" pop-up menu.
	ON_COMMAND(ID_HELP_POPUP, OnHelpPopup)
	// Handlers for Ctrl+UpArrow and Ctrl+DownArrow accelerators
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRenDlg message handlers

BOOL CRenDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	m_pApp = AfxGetApp();
	int iLen, iBaseMax, iExtMax;
	CString strText;

	// Load our accelerator table (NOTE: MFC doesn't directly support accelerator tables
	// for windows that don't have menus, so we have to roll our own.
	// Accelerator keys are processed in PreTranslateMessage()
	m_hAccelTable = LoadAccelerators(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDR_ACCELERATOR1));
	SetIcon(m_hIcon, FALSE);
	// Set basic limits for text fields.
	m_txtDigits.SetLimitText(1);
	m_txtStart.SetLimitText(9);
	m_txtBaseTemplate.SetLimitText(255);
	m_txtExtTemplate.SetLimitText(255);
	// Retrieve settings from the Registry.
	m_intDigits = m_pApp->GetProfileInt("Settings", "Digits", 2);
	strText = m_pApp->GetProfileString("Settings","BaseTemplate", "*\\n");
	m_txtBaseTemplate.SetWindowText(strText);
	strText = m_pApp->GetProfileString("Settings", "ExtTemplate", "*");
	m_txtExtTemplate.SetWindowText(strText);
	// Determine minimum value for the "digits" field.
	strText.Format("%u", m_nFileCount);
	m_nMinDigits = strText.GetLength();
	if (m_intDigits < m_nMinDigits)
		m_intDigits = m_nMinDigits;
	// Initialize the spin button control.
	m_spnDigits.SetRange(m_nMinDigits, 9);
	m_spnDigits.SetPos(m_intDigits);
	// Initialize the "Start" field.
	m_intBase = 0;
	m_txtStart.SetWindowText("0");
	// Give list control a copy of our main window handle so it can
	// post its custom message when appropriate.
	m_lstFiles.m_hWndParent = m_hWnd;

	// Set up the columns in the list control.
	m_lstFiles.InsertColumn(0, "Name");
	// Make sure columns are wide enough to display their labels.
	m_intMinNameWidth = m_lstFiles.GetStringWidth("Name") + 8;
	m_lstFiles.SetColumnWidth(0, m_intMinNameWidth);
	m_intMinExtWidth = m_lstFiles.GetStringWidth("Extension") + 8;
	m_lstFiles.InsertColumn(1, "Extension");
	m_lstFiles.SetColumnWidth(1, m_intMinExtWidth);
	m_intMinNewNameWidth = m_lstFiles.GetStringWidth("New Name") + 8;
	m_lstFiles.InsertColumn(2, "New Name");
	m_lstFiles.SetColumnWidth(2, m_intMinNewNameWidth);
	m_intLenParentFolder = m_strParentFolder.GetLength();
	
	// Create array of pointers for new name strings.
	m_pNewNames = new char*[m_nFileCount];
	if (m_pNewNames == NULL)
	{
		AfxMessageBox(IDP_OUT_OF_MEM, MB_ICONINFORMATION);
		EndDialog(IDOK);
	}
	else
	{
		// Set current minimum widths.
		iBaseMax = m_intMinNameWidth;
		iExtMax = m_intMinExtWidth;
		// Fill the list.
		for (UINT i = 0; i < m_nFileCount; i++)
		{
			m_pNewNames[i] = NULL;				// Make sure new name string pointer is NULL.
			m_lstFiles.InsertItem(LVIF_TEXT | LVIF_PARAM, i, LPSTR_TEXTCALLBACK, 0, 0, 0, (LPARAM) i);
			iLen = m_lstFiles.GetStringWidth(m_pFileNames[i]);
			if (iLen > iBaseMax)
				iBaseMax = iLen;
			iLen = m_lstFiles.GetStringWidth(m_pFileExtensions[i]);
			if (iLen > iExtMax)
				iExtMax = iLen;
		}
		// Set column widths.
		m_lstFiles.SetColumnWidth(0, iBaseMax + 16);
		m_lstFiles.SetColumnWidth(1, iExtMax + 16);
		// Make sure first item in list is selected.
		m_lstFiles.SetItemState(0, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
	}
	// Build new names and add them to the list.
	Set_New_Names();
	return TRUE;
}

// Parses a wildcard string.
// Input:
//    pszOld  = pointer to original name string.
//    pszWild = pointer to wildcard string.
//    pszNew  = pointer to buffer for resulting string,
//				which must be at least (_MAX_PATH) characters long.
//    pszSequence = pointer to sequence number string.
// Return Value:
//		TRUE = successful
//		FALSE = Resulting filename too long!
UINT CRenDlg::Parse_Wildcard(LPCTSTR pszOld, LPCTSTR pszWild, LPCTSTR pszSequence, char * pszNew)
{
	UINT nOld = 0, nWild = 0, nNew = 0, nSequence = 0, nResult = 0;
	BOOL bInsert = FALSE, bForceUpper = FALSE, bForceLower = FALSE;
	char chOld, chWild;

	chWild = pszWild[nWild];
	while ((chWild != '\0') && nResult == 0)
	{
		switch(chWild)
		{
		case '?':								// "?" Wildcard
			if (pszOld[nOld])
			{
				chOld = pszOld[nOld++];
				if (bForceUpper)
					chOld = toupper(chOld);
				else if (bForceLower)
					chOld = tolower(chOld);
				pszNew[nNew++] = chOld;
			}
			else
				pszNew[nNew++] = ' ';
			break;
		case '*':								// "*" Wildcard
			chOld = pszOld[nOld];
			while (chOld != 0)
			{
				if (bForceUpper)
					chOld = toupper(chOld);
				else if (bForceLower)
					chOld = tolower(chOld);
				pszNew[nNew++] = chOld;
				if (nNew >= _MAX_PATH)
				{
					nResult = IDP_NAME_TOO_LONG;
					break;
				}
				chOld = pszOld[++nOld];
			}
			break;
		case '\\':								// Escape character
			chWild = pszWild[++nWild];			// Get next char.
			switch (chWild)
			{
			case '-':							// "\-" = skip a character from original name.
				if (pszOld[nOld])				// If not at end of string,
					++nOld;						// skip the character.
				break;
			case 'n':							// "\n" 
			case 'N':
				while (pszSequence[nSequence] != 0)
				{
					pszNew[nNew++] = pszSequence[nSequence++];
					if (!bInsert)
					{
						if (pszOld[nOld])
							++nOld;
					}
					if (nNew >= _MAX_PATH)
					{
						nResult = IDP_NAME_TOO_LONG;
						break;
					}
				}
				nSequence = 0;
				break;
			case 'i':
			case 'I':
				bInsert = !bInsert;
				break;
			case 'u':
			case 'U':
				if ((bForceUpper = !bForceUpper))
					bForceLower = FALSE;
				break;
			case 'l':
			case 'L':
				if ((bForceLower = !bForceLower))
					bForceUpper = FALSE;
				break;
			case '\0':
				--nWild;
			default:
				nResult = IDP_INVALID_OP;
			}
			break;
		default:
			pszNew[nNew++] = pszWild[nWild];
			if (!bInsert)
				if (pszOld[nOld] != 0)
					++nOld;
		}
		chWild = pszWild[++nWild];
		if (nNew >= _MAX_PATH)
		{
			nResult = IDP_NAME_TOO_LONG;
			break;
		}
	}
	if (nResult == IDP_NAME_TOO_LONG)
		--nNew;
	pszNew[nNew] = '\0';
	return nResult;
}

// Builds new filenames.
// Returns TRUE if no files were deleted from the list
// because they no longer exist with their original names.
// Otherwise, returns FALSE.
BOOL CRenDlg::Set_New_Names()
{
	UINT			i, j, iItemCount, iLen, iMax, nWhich, nResult;
	char			szNewName[_MAX_PATH];
	CString			strNewName, strSequence, strBase, strExt, strScratch, strOldName;
	BOOL			bDupeOnDisk, bDupeInList, bNameTooLong, bDeleted, bBlank;
	CFileStatus		rStatus;

	// Nothing to do if list is empty.
	if ((iItemCount = m_lstFiles.GetItemCount()) == 0)
		return TRUE;
	// Initialize error flags.
	bDupeOnDisk = bDupeInList = bNameTooLong = bDeleted = bBlank = FALSE;
	nResult = 0;
	iMax = m_intMinNewNameWidth;			// Set minimum column width.
	m_txtDigits.GetWindowText(strScratch);	// Get text from the digits field.
	if (strScratch.IsEmpty())				// If it's empty,

	{
		m_spnDigits.SetPos(m_nMinDigits);	// force minimum setting.
		m_intDigits = m_nMinDigits;
		strScratch.Format("%u", m_intDigits);
		m_txtDigits.SetWindowText(strScratch);
	}
	else
		m_intDigits = atoi(strScratch);	// Otherwise, retrieve the setting.
	m_spnDigits.SetPos(m_intDigits);	// Make sure spin control is in synch.
	if (m_intDigits < m_nMinDigits)		// If setting is too low,
	{
		m_intDigits = m_nMinDigits;		// Force minimum and present warning.
		strScratch.Format("You must use least %u digits\n"
						"to rename %u files!", m_nMinDigits, iItemCount);
		AfxMessageBox(strScratch, MB_ICONINFORMATION);
		m_spnDigits.SetPos(m_intDigits);
	}
	m_txtStart.GetWindowText(strScratch);		// Retrieve start value.
	m_intBase = atoi(strScratch);
	nWhich = (UINT) pow(10, m_intDigits);		// Calculate max file count for # of digits.
	if (m_intBase + m_nFileCount > nWhich)		// If start value is too large,
	{
		m_intBase = nWhich - m_nFileCount;		// Calculate and force maximum setting.
		strScratch.Format("%u", m_intBase);
		m_txtStart.SetWindowText(strScratch);
		strScratch.Format("To rename %u files using %u-digit numbers,\n"
			"the start value cannot exceed %u.", iItemCount, m_intDigits, m_intBase);
		AfxMessageBox(strScratch, MB_ICONINFORMATION);
	}
	m_txtBaseTemplate.GetWindowText(strBase);	// Retrieve template strings.
	m_txtExtTemplate.GetWindowText(strExt);
/*
	for (i = 0; i < iItemCount; i++)
	{
		if (!Verify_File(i))
		{
			bDeleted = TRUE;
			--iItemCount;
			--i;
		}
	}
*/
	for (i = 0; i < iItemCount; i++)			// For each file in the list
	{
		// Get fully-qualified old filename.
//		strScratch = m_strParentFolder + strOldName;
//		if (!CFile::GetStatus(strScratch, rStatus))		// If file does not still exist,
		if (!Verify_File(i))
		{
//			m_lstFiles.DeleteItem(i);					// Remove it from the list.
			bDeleted = TRUE;							// Set flag for warning prompt.
			--iItemCount;								// Adjust count and index.
			--i;
			continue;									// Continue with next item in list.
		}
		nWhich = m_lstFiles.GetItemData(i);		// Retrieve offset to list item's data.
		// Get old file name.
		strOldName.Format("%s.%s", m_pFileNames[nWhich], m_pFileExtensions[nWhich]);
		strSequence.Format("%09u", i + m_intBase);		// Create sequence number string.
		strSequence = strSequence.Right(m_intDigits);
		// Parse the base name template, quit if syntax is invalid.
		if ((nResult = Parse_Wildcard(m_pFileNames[nWhich], strBase, strSequence, szNewName)) == IDP_INVALID_OP)
			break;
		strNewName = szNewName;	// Append trailing ".".
		strNewName += '.';
		// Parse the extension template, quit if syntax is invalid.
		if ((nResult = Parse_Wildcard(m_pFileExtensions[nWhich], strExt, strSequence, szNewName)) == IDP_INVALID_OP)
			break;
		// Append extension to new name.
		strNewName += szNewName;
		iLen = strNewName.GetLength();
		// Check length of name + parent folder path.
		if (iLen == 1)			// If name is just a ".", it's blank!
		{
			bBlank = TRUE;
			strNewName = "<<BLANK>>";
		}
		else if ((iLen + m_intLenParentFolder) >= _MAX_PATH)	// If name is too long,
		{
			strNewName = "<<NAME TOO LONG>>";					// Indicate so in the list and
			bNameTooLong = TRUE;								// set flag for error message.
		}
		if (m_pNewNames[nWhich] != NULL)		// If it exists,
			delete [] m_pNewNames[nWhich];		// delete the current string.
		iLen = strNewName.GetLength() + 1;		// Get length of new string.
		m_pNewNames[nWhich] = new char[iLen];	// Allocate memory for it.
		lstrcpyn(m_pNewNames[nWhich], strNewName, iLen);	// Copy new name to it.
		iLen = m_lstFiles.GetStringWidth(strNewName) + 16;	// Calculate necessary column width.
		if (iLen > iMax)									// Set new maximum width if appropriate.
			iMax = iLen;
		m_lstFiles.Update(i);	// Tell list control to redraw the item.
		// If new name is valid,
		if (!bNameTooLong)
		{
			// See if file already exists.
			strScratch.Format("%s%s", m_strParentFolder, m_pNewNames[nWhich]);
			if (File_Exists(strScratch))
			{
				// If old and new names are different (aside from upper/lower case),
				if (strOldName.CompareNoCase(m_pNewNames[nWhich]) != 0) 
					// Set dupe on disk flag.
					bDupeOnDisk = TRUE;
			}
		}
	}
	if (nResult == IDP_INVALID_OP)	// Syntax error?
	{
		AfxMessageBox(IDP_INVALID_OP, MB_ICONEXCLAMATION);	// Give message
		m_bNewNamesValid = FALSE;							// Reset "valid" flag.
		m_cmdSaveSettings.EnableWindow(FALSE);				// Disable "Save settings" button.
	}
	else
	{
		m_cmdSaveSettings.EnableWindow(TRUE);				// Enable "Save Settigns" button.
		m_lstFiles.SetColumnWidth(2, iMax);					// Set column width.
		// Check list for duplicate names, set flag if found.
		for (i = 1; (i < m_nFileCount) && (!bDupeInList); i++)
		{
			if (*m_pNewNames[i] != '<')
			{
				for (j = 0; j < i;  j++)
				{
					if (*m_pNewNames[j] != '<')		// Ignore "<<NAME TOO LONG>>" entries.
					{
						if (lstrcmpi(m_pNewNames[i], m_pNewNames[j]) == 0)
						{
							bDupeInList = TRUE;
							break;
						}
					}
				}
			}
		}
		// Set text for status line.
		if (bBlank)
		{
			m_bNewNamesValid = FALSE;
			strScratch = "ERROR: One or more names are blank.";
		}
		else if (bDupeInList || bDupeOnDisk)
		{
			m_bNewNamesValid = FALSE;
			strScratch = "ERROR: ";
/*
			strScratch = "ERROR: Names are duplicated ";
			if (bDupeOnDisk)
			{
				strScratch += "on disk";
				if (bDupeInList)
					strScratch += " and ";
			}
			if (bDupeInList)
				strScratch += "in list";
			strScratch += ".";
*/
			if (bDupeInList && bDupeOnDisk)
			{
				strScratch += "Names duplicated in list and on disk.";
			}
			else if (bDupeInList)
			{
				strScratch += "Two or more duplicate names in list.";
			}
			else
			{
				strScratch += "Conflict with existing file or system device names.";
			}
		}
		else if (bNameTooLong)
		{
			m_bNewNamesValid = FALSE;
			strScratch = "ERROR: One or more names are too long.";
		}
		else
		{
			m_bNewNamesValid = TRUE;
			strScratch.Format("Parent Folder: %s", m_strParentFolder);
		}
		m_lblWarning.SetWindowText(strScratch);
	}
	if (bDeleted)
	{
		if (iItemCount)
			AfxMessageBox(IDP_WARN_DELETED, MB_ICONINFORMATION);
		else
		{
			AfxMessageBox(IDP_WARN_ALL_DELETED, MB_ICONSTOP);
			EndDialog(IDOK);
		}
	}
	// If names are not valid, disable "Apply New Names" button.
	m_cmdExecute.EnableWindow(m_bNewNamesValid);
	// Reset synchronization flag.
	m_bSettingsChanged = FALSE;
	return !bDeleted;
}

// "Preview" button.
void CRenDlg::OnOK() 
{
	Set_New_Names();
	return;
}

// Called when tree control needs data for an item.
void CRenDlg::OnGetdispinfolstFiles(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;

	// item.iSubItem element tells us which column.
	switch (pDispInfo->item.iSubItem)
	{
	case 0:		// Name
		pDispInfo->item.pszText = m_pFileNames[pDispInfo->item.lParam];
		break;
	case 1:		// Extension
		pDispInfo->item.pszText = m_pFileExtensions[pDispInfo->item.lParam];
		break;
	case 2:		// New Name
		pDispInfo->item.pszText = m_pNewNames[pDispInfo->item.lParam];
		break;
	default:	// Should never be called, as we only have 3 columns.
		pDispInfo->item.pszText = NULL;
	}
	*pResult = 0;
}

// Called when user clicks a button at the top of a column.
void CRenDlg::OnColumnclicklstFiles(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	*pResult = 0;
	m_nWhichColumn = pNMListView->iSubItem;			// Which column?
	if (m_nWhichColumn > 1)							// Done if not 0 or 1
		return;
	m_lstFiles.SortItems(Compare_List_Items, (LPARAM) this);	// Sort the column.
	if (m_nWhichColumn == 0)						// Toggle appropriate sort order flag.
		m_bNameAscending = !m_bNameAscending;
	else
		m_bExtAscending = !m_bExtAscending;
	Set_New_Names();
}


// Called when user changes the spin control.
// This simply mimics clicking the "Preview" button immediately afterward.
void CRenDlg::OnDeltaposspnDigits(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;

	*pResult = 0;
	PostMessage(WM_COMMAND, IDOK, 0);
}

// "Apply New Names" command.
void CRenDlg::OncmdExecute() 
{
	UINT i, iItemCount, nWhich;
	char szOldName[_MAX_PATH], szNewName[_MAX_PATH];
	char *pOldName, *pNewName, *pSource, *pDest;
	BOOL bFail = FALSE;

	if (m_bSettingsChanged)			// If controls have changed,
	{
		Set_New_Names();			// Build new list and present prompt.
		AfxMessageBox(IDP_WARN_SETTING_CHANGED, MB_ICONINFORMATION);
		return;
	}
	else
	{
		if (!Set_New_Names())
			return;
	}
	// Display appropriate confirmation prompt.
	if (m_bIsSystemFolder)
	{
		if (AfxMessageBox(IDP_WARN_SYSTEM_FOLDER, MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) != IDYES)
			return;
	}
	else if (m_bExecutablesInList)
	{
		if (AfxMessageBox(IDP_WARN_EXECUTABLE, MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) != IDYES)
			return;
	}
	else
	{
		if (AfxMessageBox(IDP_WARN_DEFAULT, MB_ICONQUESTION | MB_YESNO) != IDYES)
			return;
	}
	lstrcpy(szOldName, m_strParentFolder);
	lstrcpy(szNewName, szOldName);
	pOldName = szOldName + m_intLenParentFolder;
	pNewName = szNewName + m_intLenParentFolder;
	iItemCount = m_lstFiles.GetItemCount();
	// Create array to hold rename command results.
	BOOL *bFlags = new BOOL[iItemCount];
	for (i = 0; i < iItemCount; i++)
	{
		// Use the list box so we don't try to rename files that
		// we know do not exist.
		nWhich = m_lstFiles.GetItemData(i);
		// Build fully qualified names.
		pDest = pOldName;
		pSource = m_pFileNames[nWhich];
		while (*pSource) *pDest++ = *pSource++;
		*pDest++ = '.';
		pSource = m_pFileExtensions[nWhich];
		while (*pSource) *pDest++ = *pSource++;
		*pDest = '\0';
		pDest = pNewName;
		pSource = m_pNewNames[nWhich];
		while (*pSource) *pDest++ = *pSource++;
		*pDest = '\0';
		// Try to rename and set flags accordingly.
		bFail = bFail || !(bFlags[i] = Rename_File(szOldName, szNewName));
	}
	// If any renames failed, give user choice to restore original names.
	if (bFail)
	{
		if (AfxMessageBox(IDP_WARN_REN_FAIL, MB_YESNO | MB_ICONQUESTION) == IDYES)
		{
			for (i = 0; i < iItemCount; i++)
			{
				if (bFlags[i])
				{
					nWhich = m_lstFiles.GetItemData(i);
					// Build fully qualified names.
					pDest = pNewName;
					pSource = m_pFileNames[nWhich];
					while (*pSource) *pDest++ = *pSource++;
					*pDest++ = '.';
					pSource = m_pFileExtensions[nWhich];
					while (*pSource) *pDest++ = *pSource++;
					*pDest = '\0';
					pDest = pOldName;
					pSource = m_pNewNames[nWhich];
					while (*pSource) *pDest++ = *pSource++;
					*pDest = '\0';
					Rename_File(szOldName, szNewName);
				}
			}
			delete [] bFlags;	// Delete the flags array.
			return;				// Return to the dialog.
		}
	}
	else
	{
		UpdateData(TRUE);
		if (m_bUndo)
		{
			CStdioFile outfile;
			CString strBatFile, strCommand;
			BOOL bWriteFail = FALSE;

			strBatFile.Format("%sMRUndo$$.bat", m_strParentFolder);
			if (outfile.Open(strBatFile, CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive | CFile::typeText, NULL))
			{
				outfile.WriteString("@Echo Off\n"
									"If Exist MRUndo$$.BAT Goto Proceed\n"
									"Echo ERROR: You must execute this file from the same\n"
									"Echo directory in which it resides.\n"
									"Goto End\n"
									":Proceed\n"
									"Echo This .BAT file was created by MultiRen.\n"
									"Echo Running it will restore the original names\n"
									"Echo of those files processed by the most recent\n"
									"Echo MultiRen command in this directory.\n"
									"Echo.\n"
									"Echo If you do not want to do this, press Ctrl+C now, otherwise\n"
									"PAUSE\n"
									"@Echo On\n");
				for (i = 0; i < iItemCount; i++)
				{
					nWhich = m_lstFiles.GetItemData(i);
					strCommand.Format("REN \"%s\" \"%s.%s\"\n",
						m_pNewNames[nWhich], m_pFileNames[nWhich], m_pFileExtensions[nWhich]);
					if (strCommand.GetLength() > 127)
					{
						strCommand.Format(
							"@Echo off\n"
							"@Echo.\n"
							"Echo ** Cannot rename \"%s\"\n"
							"Echo ** to \"%s.%s\"\n"
							"Echo ** -- command line is too long.\n"
							"Echo.\n"
							"@Echo on\n",
							m_pNewNames[nWhich], m_pFileNames[nWhich], m_pFileExtensions[nWhich]);
					}
					try
					{
						outfile.WriteString(strCommand);
					}
					catch (CFileException *e)
					{
						e->Delete();
						AfxMessageBox("Error writing to MRUndo$$.BAT");
						bWriteFail = TRUE;
						break;
					}
				}
				if (!bWriteFail)
					outfile.WriteString(":End\n");
				outfile.Close();
				if (bWriteFail)
				{
					_unlink(strBatFile);
				}
			}
			else
			{
				AfxMessageBox("Unable to create MRUndo$$.BAT");
			}
		}
	}
	delete [] bFlags;		// Delete the flags array.
	EndDialog(IDOK);		// All done, so close the dialog.
}

// Handler for list control's sort command.
// Note: This is a static function, so it needs the "this" pointer
// passed via the nData parameter to be able to access the dialog's data.
int CALLBACK CRenDlg::Compare_List_Items(LPARAM lParam1, LPARAM lParam2, LPARAM nData)
{
	int iResult;
	const CRenDlg* pDlg = (CRenDlg*) nData;		// Get pointer to dialog.
	const char* const * pItemArray;
	BOOL bFlag;
	
	if (pDlg->m_nWhichColumn == 1)				// Sorting extension column?
	{
		pItemArray = pDlg->m_pFileExtensions;	// Get pointer to extensions array
		bFlag = pDlg->m_bExtAscending;			// Retrieve sort order flag for extension column.
	}
	else										// Otherwise, sorting base names, so
	{
		pItemArray = pDlg->m_pFileNames;		// Get pointer to base name array.
		bFlag = pDlg->m_bNameAscending;			// Retrieve sort order flag for base name column.
	}
	iResult = lstrcmp(pItemArray[lParam2], pItemArray[lParam1]);	// Compare the items.
	return (bFlag ? -iResult : iResult);
}

// Handles cleanup at shutdown time.
BOOL CRenDlg::DestroyWindow()
{
	// Delete data for new names array.
	if (m_pNewNames != NULL)
	{
		for (UINT i = 0; i < m_nFileCount; i++)
		{
			if (m_pNewNames[i] != NULL)
				delete [] m_pNewNames[i];
		}
		delete [] m_pNewNames;
	}
	// Make sure WinHelp shuts down unless another app is using it.
	if (m_pApp)
		m_pApp->WinHelp(0, HELP_QUIT);
	return CDialog::DestroyWindow();
}

// Called when user changes the contents of any of our edit controls.
// The m_bSettingsChanged flag is used by OncmdExecute()
// to determine whether or not the settings have changed since they were last applied.
// When Set_New_Names() succeeds in generating a set of valid names,
// the m_bSettingsChanged flag is cleared, so OncmdExecute() knows that
// the settings and name list are in synch.
void CRenDlg::OnChangetxtBaseTemplate() 
{
	m_bSettingsChanged = TRUE;
}


// Write the filename templates and # of digits to the Registry.
void CRenDlg::OncmdSaveSettings() 
{
	CString strText;

	if (m_bSettingsChanged)				// If controls aren't in synch,
	{
		Set_New_Names();				// Synch them.
		if (!m_bNewNamesValid)			// If templates are invalid, don't save them!
		{
			AfxMessageBox(IDP_BAD_TEMPLATE, MB_ICONINFORMATION);
			return;
		}
	}
	m_txtBaseTemplate.GetWindowText(strText);
	m_pApp->WriteProfileString("Settings", "BaseTemplate", strText);
	m_txtExtTemplate.GetWindowText(strText);
	m_pApp->WriteProfileString("Settings", "ExtTemplate", strText);
	m_pApp->WriteProfileInt("Settings", "Digits", m_intDigits);
}

// Handles message posted by the list control when user
// drags an item to a new position.
LONG CRenDlg::OnMyItemMoved(WPARAM, LPARAM)
{
	Set_New_Names();
	return 0L;
}


// Called when user right-clicks the dialog, outside
// any controls (such as text boxes) that have their own
// context menus.
void CRenDlg::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	CPoint ptThis = point, ptTest = point;
	CWnd * pWndTarget = NULL;
	int iCtrlID = 0, iItem, iLastItem;

	ScreenToClient(&ptThis);		// Convert to client coordinates.
	// Which window was he pointing at.
	pWndTarget = ChildWindowFromPoint(ptThis, CWP_SKIPTRANSPARENT);
	// Make sure the window belongs to us and is _not_ the dialog itself.
	if ((pWndTarget != NULL) && (pWndTarget != this))
	{
		// Get identifier of control pointed to.
		iCtrlID = pWndTarget->GetDlgCtrlID();
		// If it's not one of the "standard buttons",
		if (iCtrlID != IDCANCEL && iCtrlID != ID_HELP)
		{
			// Set popup help context from the control ID.
			// Note: Popup topic ID's are created by MAKEHELP.BAT
			m_nPopupTopic = iCtrlID + 0x60000;
			// Create the popup menu and add the "What's This?" command.
			CMenu mnu;
			mnu.CreatePopupMenu();
			mnu.AppendMenu(MF_STRING, ID_HELP_POPUP, "&What's This?");
			// Was user pointing at the list box?
			if (iCtrlID == IDC_lstFiles)
			{
				// Convert to list box client coordinates.
				m_lstFiles.ScreenToClient(&ptTest);
				// Get index of last item in the list.
				iLastItem = m_lstFiles.GetItemCount() - 1;
				// See if user was pointing at an item in the list.
				iItem = m_lstFiles.HitTest(ptTest);
				// If so,
				if (iItem >= 0)
				{
					// Add a separator to the menu.
					mnu.AppendMenu(MF_SEPARATOR);
					// If target item is not already at the top of the list,
					if (iItem > 0)
						// add the "move up" command to the menu.
						mnu.AppendMenu(MF_STRING, ID_MOVE_UP, "Move file &up");
					// If target item is not already at the end of the list,
					if (iItem < iLastItem)
						// add the "move down" command to the menu.
						mnu.AppendMenu(MF_STRING, ID_MOVE_DOWN, "Move file &down");
				}
			}
			// Launch the menu.
			mnu.TrackPopupMenu(TPM_CENTERALIGN | TPM_LEFTBUTTON, point.x, point.y, this, NULL);
		}
	}
}

// Called when user selects "What's This?" from the context menu.
void CRenDlg::OnHelpPopup()
{
	WinHelp(m_nPopupTopic, HELP_CONTEXTPOPUP);
}

// Called when window is being resized.
void CRenDlg::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	CRect rect(0, 0, 276, 234);		// Minimum dimensions of client area in DLUs.

	MapDialogRect(&rect);			// Convert DLUs to screen units (pixels).
	CalcWindowRect(&rect);			// Calculate bounding rectangle from client rectangle.
	// Set minimum tracking size.
	lpMMI->ptMinTrackSize.x = rect.Width();
	lpMMI->ptMinTrackSize.y = rect.Height();
}

// Called when user resizes the dialog.
void CRenDlg::OnSize(UINT nType, int cx, int cy) 
{
	if (m_lstFiles.m_hWnd != NULL)
	{
		CRect rectWork(0, 0, 7, 2);		// Size of left and right margin in DLUs
		CRect rectFiles;

		MapDialogRect(&rectWork);						// Convert margins from DLUs to pixels.
		m_lstFiles.GetWindowRect(&rectFiles);			// Get position of file list control.
		ScreenToClient(&rectFiles);						// Convert to client coordinates.
		rectFiles.right = cx - (rectWork.right);		// Adjust right and bottom edges, allowing
		rectFiles.bottom = cy - (rectWork.bottom);		// for an appropriate margin.
		m_lstFiles.MoveWindow(rectFiles, TRUE);			// Move the control to its new position.
		m_lblWarning.GetWindowRect(&rectFiles);			// Do the same for the right edge of the status indicator.
		ScreenToClient(&rectFiles);
		rectFiles.right = cx - (rectWork.right);
		m_lblWarning.MoveWindow(rectFiles, TRUE);
	}
	CDialog::OnSize(nType, cx, cy);				// Let MFC do its thing.
}

// Simple message box will do for the "About" box.
void CRenDlg::OncmdAbout() 
{
	::MessageBox(m_hWnd, VERSION_MESSAGE_STRING
		"Copyright (C) 1998 Ziff-Davis, Inc.\n\n"
		"First published in PC Magazine, US Edition, June 9, 1998.\n\n"
		"Written by Gregory A Wolking", "About MultiRen", MB_ICONINFORMATION);
	
}

// Attempts to rename file using specified names.
// Return value indicates success (TRUE) or failure (FALSE).
BOOL CRenDlg::Rename_File(LPCTSTR pszOldName, LPCTSTR pszNewName)
{
	try
	{
		CFile::Rename(pszOldName, pszNewName);
	}
	catch (CFileException* e)
	{
		e->Delete();
		return FALSE;
	}
	return TRUE;
}

// Called by Set_New_Names()
// Verifies that original file specified by the list index
// still exists on disk. If it doesn't, it deletes that
// file from the list and adjusts the arrays and item
// count accordingly.
// Returns TRUE if file existed, FALSE if it didn't.
BOOL CRenDlg::Verify_File(int iIndex)
{
	int iItem;
	CString strFileName;

	iItem = m_lstFiles.GetItemData(iIndex);
	strFileName.Format("%s%s.%s", m_strParentFolder, m_pFileNames[iItem], m_pFileExtensions[iItem]);
	if (File_Exists(strFileName))
		return TRUE;
	m_lstFiles.DeleteItem(iIndex);
	return FALSE;
}

BOOL CRenDlg::File_Exists(LPCTSTR pszFileName)
{
	static CFileStatus rStatus;

	return CFile::GetStatus(pszFileName, rStatus);
}


// This override allows us to handle our acclerator table.
BOOL CRenDlg::PreTranslateMessage(MSG* pMsg) 
{
	// See if message is a keyboard message.
	if (pMsg->message >= WM_KEYFIRST && pMsg->message <= WM_KEYLAST)
	{
		// If it is, try to translate it to an accelerator.
		if (::TranslateAccelerator(m_hWnd, m_hAccelTable, pMsg))
			return TRUE;	// If call succeeds, we're done.
	}
	// Message is not an accelerator keystroke, so process it normally.
	return CDialog::PreTranslateMessage(pMsg);
}

void CRenDlg::OnMoveUp() 
{
	int iItem, iCount, i;
	DWORD nData;
	CWnd *pWnd = NULL;

	pWnd = GetFocus();
	if (pWnd != NULL)
	{
		if (pWnd->GetDlgCtrlID() != IDC_lstFiles)
			return;
	}
	else
		return;
	iCount = m_lstFiles.GetItemCount();		// Number of items to search.
	iItem = -1;								// Default = no items found.
	for (i = 1; i < iCount; i++)			// Search starts with 2nd item in list.
	{
		if (m_lstFiles.GetItemState(i, LVIS_SELECTED) == LVIS_SELECTED)	// Is this item selected?
		{
			iItem = i;						// Save its index.
			break;							// Done searching.
		}
	}
	if (iItem <= 0)		// Nothing to do if no items selected, or selected item is already at top of list.
		return;
	nData = m_lstFiles.GetItemData(iItem);		// Retrieve item's data.
	m_lstFiles.EnsureVisible(iItem, FALSE);		// Make sure item is visible.
	m_lstFiles.DeleteItem(iItem);				// Delete item from its old position.
	--iItem;									// Set item's new position.
	// Insert item at desired position, and make sure it's both visible and selected.
	m_lstFiles.InsertItem(LVIF_TEXT | LVIF_PARAM, iItem, LPSTR_TEXTCALLBACK, 0, 0, 0, (LPARAM) nData);
	m_lstFiles.EnsureVisible(iItem, FALSE);
	m_lstFiles.SetItemState(iItem, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
	Set_New_Names();	// Rebuild name list to reflect new item order.
}

void CRenDlg::OnMoveDown() 
{
	int iItem, iCount, i;
	DWORD nData;
	CWnd* pWnd = NULL;

	pWnd = GetFocus();
	if (pWnd != NULL)
	{
		if (pWnd->GetDlgCtrlID() != IDC_lstFiles)
			return;
	}
	else
		return;
	iCount = m_lstFiles.GetItemCount() - 1;		// Search through the next-to-last item in the list.
	iItem = -1;									// Default value = item not found.
	for (i = 0; i < iCount; i++)				// Search through the list.
	{
		if (m_lstFiles.GetItemState(i, LVIS_SELECTED) == LVIS_SELECTED)	// Is current item selected?
		{
			iItem = i;							// Retrieve its index.
			break;								// Done searching.
		}
	}
	if (iItem < 0)		// No items selected?
		return;			// Done if so.
	nData = m_lstFiles.GetItemData(iItem);		// Retrieve item's data.
	m_lstFiles.EnsureVisible(iItem, FALSE);		// Make sure the item is visible.
	m_lstFiles.DeleteItem(iItem);				// Delete it.
	++iItem;									// Item moves to next position in the list.
	// Insert the item, then make sure it is both visible and selected.
	m_lstFiles.InsertItem(LVIF_TEXT | LVIF_PARAM, iItem, LPSTR_TEXTCALLBACK, 0, 0, 0, (LPARAM) nData);
	m_lstFiles.EnsureVisible(iItem, FALSE);
	m_lstFiles.SetItemState(iItem, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
	Set_New_Names();	// Rebuild name list to reflect new item order.
}
